/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.commonnames;

import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.RoleFactory;
import it.tidalwave.util.Task;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.capabilitiesprovider.ThreadLookupBinder;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.mobile.preferences.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.spi.TaxonFactSheetViewControllerSupport;
import it.tidalwave.role.ui.spi.ActionProviderSupport;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultTaxonCommonNamesFactSheetViewController extends TaxonFactSheetViewControllerSupport<TaxonCommonNamesFactSheetView> 
                                                            implements TaxonCommonNamesFactSheetViewController
  {
    private final Provider<TaxonomyPreferences> taxonomyPreferences = Locator.createProviderFor(TaxonomyPreferences.class);
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    class ShowCommonNameInFullScreenAction extends AbstractAction
      {
        @Nonnull
        private final PresentationModel presentationModel;

        public ShowCommonNameInFullScreenAction (final @Nonnull PresentationModel presentationModel)
          {
            super("Show in full screen"); // FIXME i18n
            this.presentationModel = presentationModel;
          }
                
        public void actionPerformed (final @Nonnull ActionEvent event) 
          {
            showCommonNameInFullScreen(presentationModel);
          }        
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    class CommonNamesActionProvider extends ActionProviderSupport
      {
        @Nonnull
        private final ShowCommonNameInFullScreenAction showCommonNameInFullScreenAction;
        
        public CommonNamesActionProvider (final @Nonnull PresentationModel presentationModel) 
          {
            showCommonNameInFullScreenAction = new ShowCommonNameInFullScreenAction(presentationModel);
          }

        @Override @Nonnull
        public Collection<? extends Action> getActions()
          {
            return Collections.singletonList(showCommonNameInFullScreenAction);
          }
                
        @Override @Nonnull
        public Action getDefaultAction() 
          {
            return showCommonNameInFullScreenAction;
          }        
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final RoleFactory<CommonNamesPresentationModel> actionProviderFactory = new RoleFactory<CommonNamesPresentationModel>()
      {
        @Nonnull
        public Object createRoleFor (final @Nonnull CommonNamesPresentationModel presentationModel) 
          {
            return new CommonNamesActionProvider(presentationModel);
          }
      };
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultTaxonCommonNamesFactSheetViewController (final @Nonnull TaxonCommonNamesFactSheetView view)
      {
        super(view);        
        roleRegister.registerRoleFactory(actionProviderFactory).forClass(CommonNamesPresentationModel.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void createPresentationModels (final @Nonnull Taxon taxon)
      {
        ThreadLookupBinder.with(roleRegister).run(new Task<Void, RuntimeException>() 
          {
            public Void run() 
              {
                final List<CommonNamesPresentationModel> presentationModels = new ArrayList<CommonNamesPresentationModel>();

                for (final Locale locale : taxonomyPreferences.get().getTaxonomy().getLocales())
                  {
                    try
                      {
                        presentationModels.add(new CommonNamesPresentationModel(taxon, locale));
                      }
                    catch (NotFoundException e)
                      {
                        // ok, no name FIXME: log a warning
                      }
                  }

                Collections.sort(presentationModels, new Comparator<CommonNamesPresentationModel>() 
                  {
                    public int compare (final @Nonnull CommonNamesPresentationModel o1, 
                                        final @Nonnull CommonNamesPresentationModel o2) 
                      {
                        return o1.getLanguage().compareTo(o2.getLanguage());
                      }
                  });

                for (final PresentationModel presentationModel : presentationModels) 
                  {
                    addPresentationModel(presentationModel);
                  }
                
                return null;
              }
          });          
      }
    
    /*******************************************************************************************************************
     *
     * 
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void showCommonNameInFullScreen (final @Nonnull PresentationModel presentationModel)
      {        
      }
  }
