/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://kenai.com/hg/bluebill~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.factsheet.stillimage;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import it.tidalwave.util.Id;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.Task;
import it.tidalwave.util.thread.ThreadType;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.netbeans.capabilitiesprovider.ThreadLookupBinder;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.mobile.taxonomy.factsheet.spi.TaxonFactSheetViewControllerSupport;
import static it.tidalwave.role.ui.Presentable.Presentable;
import static it.tidalwave.semantic.vocabulary.FoafVocabulary.*;
import static it.tidalwave.mobile.media.Media.*;
import static it.tidalwave.bluebill.factsheet.Documentable.Documentable;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class DefaultTaxonStillImageFactSheetViewController extends TaxonFactSheetViewControllerSupport<TaxonStillImageFactSheetView> 
                                                           implements TaxonStillImageFactSheetViewController
  {
    private static final Logger log = LoggerFactory.getLogger(DefaultTaxonStillImageFactSheetViewController.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DefaultTaxonStillImageFactSheetViewController (final @Nonnull TaxonStillImageFactSheetView view)
      {
        super(view);
      }
       
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=ThreadType.NOT_UI)
    protected void createPresentationModels (final @Nonnull Taxon taxon) 
      throws NotFoundException
      {
        ThreadLookupBinder.with(roleRegister).run(new Task<Void, NotFoundException>() 
          {
            public Void run() 
              throws NotFoundException
              {
        //                    stillImages.addAll(taxon.as(Documentable).getFactSheet().getMultiple(FOAF_IMAGE));
                for (final Media stillImage : taxon.as(Documentable).getFactSheet().getMultiple(FOAF_IMAGE))
                  {
                    addPresentationModel(stillImage.with(ID, replacedId(stillImage.get(ID))).as(Presentable).createPresentationModel());                      
                  }
                
                return null;
              }
          });
      }    
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void showImage (final @Nonnegative int index) 
      {
        log.info("showImage({})", index);
        
        view.renderImage(getPresentationModel(index));
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private Id replacedId (final @Nonnull Id id)
      {
        String x = id.stringValue();

        if (x.startsWith("http://upload.wikimedia.org")) // FIXME
          {
            x = "http://bluebill.tidalwave.it/media/1/480/" + x.substring(7); /// FIXME: pick the proper size
          }
        
        return new Id(x);
      }
  }
